------------------------
-- SpaceXplorer
-- main.lua
-- Main program
-- credit:
--   BG Music: https://www.playonloop.com/2015-music-loops/stealth-mode/
------------------------

-- koordinat
local centerX
local centerY
local shipAnchorY
local height
local width

-- musik dan font
local bgmusic
local font

-- reference ke game object
local ship
local bgspace 
local canon
local stars
local asteroids
local explosion

-- Head Up Display + button
local score
local hiscore
local gameover
local gameoverImg
local playImg

-- cek collision
local function checkCollision()
  for i = #asteroids.rocks, 1, -1 do
    for j = #canon.missile, 1, -1 do
      -- peluru dan asteroid
      local dx = canon.missile[j].x - asteroids.rocks[i].x
      local dy = canon.missile[j].y - asteroids.rocks[i].y
      local d = math.sqrt(math.pow(dx, 2) + math.pow(dy, 2))
      
      if d <= 16 then
        explosion.bang(asteroids.rocks[i].x, asteroids.rocks[i].y)
        canon.missile[j].life = false
        asteroids.rocks[i].life = false
        score = score + 10
        
        if score > hiscore then 
          hiscore = score 
        end
      end
    end
    
    -- asteroid and pesawat
    local dx = asteroids.rocks[i].x - ship.x
    local dy = asteroids.rocks[i].y - ship.y
    local d = math.sqrt(math.pow(dx, 2) + math.pow(dy, 2))
      
    if d <= 48 then
      explosion.bang(asteroids.rocks[i].x, asteroids.rocks[i].y)
      asteroids.rocks[i].life = false
      ship.shake()
      ship.shield = ship.shield - 1
      
      if ship.shield == 0 then
        gameover = true
        ship.moveTo(ship.x, height + 64)
      end
    end
  end
end

-- menampilkan score, hi-schore, dan shield-bar
local function showHUD()
  love.graphics.setColor(1, 1, 1, 0.7)
  love.graphics.print('score: ' .. score, 10, 10)
  love.graphics.print('hi-score: ' .. hiscore, width - 125, 10)
  
  love.graphics.print('shield', 10, height - 28)
  love.graphics.rectangle('line', 10, height - 15, 44, 5)
  love.graphics.setColor(0, 1, 0, 1)
  love.graphics.rectangle('fill', 11, height - 14, 14 * ship.shield, 3)
  love.graphics.setColor(1, 1, 1, 1)
  
  if gameover then
    love.graphics.draw(gameoverImg, centerX, centerY, 0, 1, 1, 100, 23)
    love.graphics.draw(playImg, centerX, shipAnchorY, 0, 1, 1, 24, 24)
  end
end

function resetGame()
  gameover = false
  score = 0
  ship.shield = 3
  ship.moveTo(centerX, shipAnchorY)
end

function love.load()
  width = love.graphics.getWidth()
  height = love.graphics.getHeight()
  
  centerX = width / 2
  centerY = height / 2
  
  shipAnchorY = centerY + 75
  
  bgmusic = love.audio.newSource('POL-stealth-mode-short.wav', 'stream')
  bgmusic:setLooping(true)
  
  ship = require('ship')
  ship.moveTo(centerX, shipAnchorY)
  
  canon = require('canon')
  stars = require('stars')
  asteroids = require('asteroids')
  explosion = require('explosion')
  
  font = love.graphics.newFont('whitrabt.ttf')
  love.graphics.setFont(font)
  
  bgspace = love.graphics.newImage('background.jpg')
  gameoverImg = love.graphics.newImage('gameover.png')
  playImg = love.graphics.newImage('play.png')
  
  score = 0
  hiscore = 0
  gameover = false
  
  bgmusic:play()
end

function love.update(dt)
  stars.update()
  
  if not gameover then
    asteroids.create(dt)  
    checkCollision()
  end
  
  ship.update(dt)
  canon.update()
  
  asteroids.update()
  explosion.update(dt)
end

function love.draw()
  love.graphics.setColor(1, 1, 1, 1)
  love.graphics.draw(bgspace, 0, 0)
  stars.draw()
  asteroids.draw()
  canon.draw()
  ship.draw()
  explosion.draw()
  
  showHUD()
end

function love.mousepressed(x, y, button)
  if button == 1 then
    if gameover then
      local dx = centerX - x
      local dy = shipAnchorY - y
      local d = math.sqrt(math.pow(dx, 2) + math.pow(dy, 2))
      
      if d <= 24 then
        resetGame()
      end
    else
      ship.moveTo(x, shipAnchorY)
      canon.fire(ship.x, shipAnchorY)
    end
  end
end

function love.mousemoved(x, y)
  if love.mouse.isDown(1) then
    if not gameover then
      ship.moveTo(x, shipAnchorY)
    end
  end
end
